#########################################
#   CRITICAL PRELIMINARY REQUIREMENTS
# 1. Download MixRegLS: https://hedeker-sites.uchicago.edu/sites/hedeker.uchicago.edu/files/uploads/MixregLS_RevisedSept2013.zip
# 2. Unzip MIXREGLS.zip folder (critical files in the folder: mixregls.dll, mixregls_function.r, & mixregls_run.r)
# 3. Note directory for unzipped MIXREGLS folder
#########################################
  
#Install Formula (required to run MixRegLS)
#install.packages(c("Formula","LaplacesDemon"))
#library(Formula)
#library(LaplacesDemon)

###############################################################
###            Define MACRO Variable Values Below           ###
###      Use Calculate_Values.xlsx to Determine Values      ###
###############################################################

#Folder Directory for (unzipped) MixRegLS (note the / before the second ")
  MIXREGLS = "E:/Users/walters/Desktop/MIXREGLS/"
#Individuals
  N = 50
#Repeated Occasions
  ni = 15
#Location-Model Fixed Intercept (Outcome Scale)
  beta0 = 0
#Location-Model Random Intercept Variance (Outcome Scale)
  b0var = 2
#Scale-Model Fixed Intercept (Log Scale)
  tau0 = 0.643
#Scale-Model Random Intercept Variance
  t0var = 0.10
#Correlation between Location- and Scale-Model Random Intercept
  b0t0corr = 0.30
#Number of Replications for Power Analysis
  Nreps = 1000

################################################
###      !DO NOT CHANGE ANYTHING BELOW!      ###
###   Create Data, Run MIXREGLS, Get Power   ###
################################################
  
#Load MIXREGLS dll (Make sure you're using a 32-bit version of R)
#You may get a warining that DLL attempted to change FPU from 8001f to 9001f (no worries...)
dyn.load(paste(MIXREGLS,"mixregls.dll",sep=""))
is.loaded("mixreglsr")
source(paste(MIXREGLS,"mixregls_function.r",sep=""))

#Calculate Clopper-Pearson (exact) CIs
ClopPears <- function(x,n,alpha){
  if (x == 0) {
    ll <- 0
    ul <- 1 - (alpha/2)^(1/n)
  }
  else if (x == n) {
    ll <- (alpha/2)^(1/n)
    ul <- 1
  }
  else {
    ll <- 1/(1 + (n - x + 1) / (x * qf(alpha/2, 2 * x, 2 * (n-x+1))))
    ul <- 1/(1 + (n - x) / ((x + 1) * qf(1-alpha/2, 2 * (x+1), 2 *
                                           (n-x))))
  }
  c(ll,ul)
}

#Create empty Results Data Frame
Results = data.frame(Rep=numeric(),
                     beta0_NS=numeric(),
                     b0var_NS=numeric(),
                     b0var_NS_DS=numeric(),
                     tau0_NS=numeric(),
                     beta0_YS=numeric(),
                     b0var_YS_DS=numeric(),
                     evar_YS_DS=numeric(),
                     ICC_NS=numeric(),
                     ICC_YS=numeric(),
                     evar_YS=numeric(),
                     b0var_YS=numeric(), 
                     tau0_YS=numeric(),
                     t0var_YS=numeric(),
                     b0t0corr_YS=numeric(),
                     Neg2LL_NS=numeric(),  
                     Neg2LL_YS=numeric(),
                     t0sd_YS=numeric(),
                     RLonWS_YS=numeric(),
                     b0t0cov_YS=numeric(),
                     t0var_SS=numeric())

for (i in 1:Nreps){
  
  #Create Data
  MELS = NULL
  rho = b0t0corr
  for (PersonID in 1:N){  
    #Generate Standard Normal Variates with Specified Correlation
    b0i = rnorm(1,mean=0,sd=1)
    t0i = rho*b0i + sqrt(1 - rho**2)*rnorm(1,mean=0,sd=1)
    #Transform Intercepts to have (Prior) Mean of 0 with Specified Variance
    b0i = 0 + sqrt(b0var)*b0i
    t0i = 0 + sqrt(t0var)*t0i
    #Create Repeated Measures
    for (MeasureID in 1:ni){
      #Create Heterogeneous Residual Variances
      evar_i = exp(tau0 + t0i)
      e_ti = rnorm(1,mean=0,sd=sqrt(evar_i))
      #Create Outcome Variable
      Y = (beta0 + b0i) + e_ti
      MELS=rbind(MELS,data.frame(PersonID, MeasureID, Y))
    }
  }

  if (b0t0corr == 0){
    #Run MixRegLS with NO location-scale correlation
    mixregls.results = mixregls.combined(Y ~ 1 | 1 | 1 , id="PersonID",data=MELS, NQ=15, NCOV=0)

    Results[i,"Rep"] = i
    Results[i,"beta0_NS"] = mixregls.results$beta1
    Results[i,"b0var_NS"] = mixregls.results$alpha1
    Results[i,"b0var_NS_DS"] = exp(mixregls.results$alpha1)
    Results[i,"tau0_NS"] = mixregls.results$tau1
    Results[i,"beta0_YS"] = mixregls.results$beta1
    Results[i,"b0var_YS"] = mixregls.results$alpha3
    Results[i,"b0var_YS_DS"] = exp(mixregls.results$alpha3)
    Results[i,"tau0_YS"] = mixregls.results$tau3
    Results[i,"t0sd_YS"] = mixregls.results$spar3
    Results[i,"t0var_YS"] = mixregls.results$spar3**2
    Results[i,"Neg2LL_NS"] = mixregls.results$dev1
    Results[i,"Neg2LL_YS"] = mixregls.results$dev3
    Results[i,"ICC_NS"] = exp(mixregls.results$alpha1) / (exp(mixregls.results$alpha1)+exp(mixregls.results$tau1))
    Results[i,"ICC_YS"] = exp(mixregls.results$alpha3) / (exp(mixregls.results$alpha3)+exp(mixregls.results$tau3 + 0.5*(mixregls.results$spar3**2)))
    if (mixregls.results$dev1 - mixregls.results$dev3 > 2.71){
      Results[i,"t0var_SS"] = 1
    } else {
      Results[i,"t0var_SS"] = 0
    }
    Results[i,"evar_YS"] = mixregls.results$tau3 + 0.5*(mixregls.results$spar3**2)
    Results[i,"evar_YS_DS"] = exp(mixregls.results$tau3 + 0.5*(mixregls.results$spar3**2))
  } else {
    #Run MixRegLS with location-scale RI correlation
    mixregls.results = mixregls.combined(Y ~ 1 | 1 | 1 , id="PersonID",data=MELS, NQ=15, NCOV=1)
    
    Results[i,"Rep"] = i
    Results[i,"beta0_NS"] = mixregls.results$beta1
    Results[i,"b0var_NS"] = mixregls.results$alpha1
    Results[i,"b0var_NS_DS"] = exp(mixregls.results$alpha1)
    Results[i,"tau0_NS"] = mixregls.results$tau1
    Results[i,"beta0_YS"] = mixregls.results$beta1
    Results[i,"b0var_YS"] = mixregls.results$alpha3
    Results[i,"b0var_YS_DS"] = exp(mixregls.results$alpha3)
    Results[i,"tau0_YS"] = mixregls.results$tau3
    Results[i,"t0sd_YS"] = mixregls.results$spar3[2,1]
    Results[i,"RLonWS_YS"] = mixregls.results$spar3[1,1]
    Results[i,"t0var_YS"] = mixregls.results$spar3[1,1]**2 + mixregls.results$spar3[2,1]**2
    Results[i,"b0t0cov_YS"] = mixregls.results$spar3[1,1]*sqrt(exp(mixregls.results$alpha3))
    Results[i,"b0t0corr_YS"] = mixregls.results$spar3[1,1]*sqrt(exp(mixregls.results$alpha3)) / (sqrt(exp(mixregls.results$alpha3)*(mixregls.results$spar3[1,1]**2 + mixregls.results$spar3[2,1]**2)))
    Results[i,"Neg2LL_NS"] = mixregls.results$dev1
    Results[i,"Neg2LL_YS"] = mixregls.results$dev3
    Results[i,"ICC_NS"] = exp(mixregls.results$alpha1) / (exp(mixregls.results$alpha1)+exp(mixregls.results$tau1))
    Results[i,"ICC_YS"] = exp(mixregls.results$alpha3) / (exp(mixregls.results$alpha3)+exp(mixregls.results$tau3 + 0.5*(mixregls.results$spar3[1,1]**2 + mixregls.results$spar3[2,1]**2)))
    if (mixregls.results$dev1 - mixregls.results$dev3 > 5.14){
      Results[i,"t0var_SS"] = 1
    } else {
      Results[i,"t0var_SS"] = 0
    }
    Results[i,"evar_YS"] = mixregls.results$tau3 + 0.5*(mixregls.results$spar3[1,1]**2 + mixregls.results$spar3[2,1]**2)
    Results[i,"evar_YS_DS"] = exp(mixregls.results$tau3 + 0.5*(mixregls.results$spar3[1,1]**2 + mixregls.results$spar3[2,1]**2))
  }
}

#Calculate Parameter Recovery
ParamRecov = data.frame(Obs=numeric(),Mean=numeric(),Lower95CI=numeric(),Upper95CI=numeric(),
                        Pctl25=numeric(),Pctl50=numeric(),Pctl75=numeric())

if (b0t0corr == 0){

  for (j in 7:14) {
    ParamRecov[(j - 6),1] = Nreps
    ParamRecov[(j - 6),2] = mean(Results[,j])
    ParamRecov[(j - 6),3] = mean(Results[,j]) - 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 6),4] = mean(Results[,j]) + 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 6),5] = quantile(Results[,j], c(.25))
    ParamRecov[(j - 6),6] = quantile(Results[,j], c(.50))
    ParamRecov[(j - 6),7] = quantile(Results[,j], c(.75))
  }
  
  rownamesobj = c("b0var: LM RI Variance (Outcome Scale) - Yes SMRI",
                  "evar: Residual Variance (Outcome Scale) - Yes SMRI",
                  "ICC: Unconditional Intra-Class Correlation - No SMRI",
                  "ICC: Unconditional Intra-Class Correlation - Yes SMRI",
                  "evar: Residual Variance (Log Scale) - Yes SMRI",
                  "b0var: LM RI Variance (Log Scale) - Yes SMRI",
                  "tau0: SM Fixed Intercept (Log Scale) - Yes SMRI",
                  "t0var: SM RI Variance (Log Scale)")
  
  rownames(ParamRecov) = rownamesobj
  
  sig = sum(Results$t0var_SS == 1)
  CI = ClopPears(sig,Nreps,0.05)
  CI = as.matrix(CI)
  Power = data.frame(Power=numeric(), Lower95CI=numeric(), Upper95CI=numeric())
  Power[1,1] = mean(Results$t0var_SS)
  Power[1,2] = CI[1,1]
  Power[1,3] = CI[2,1]
  
} else {
  
  for (j in 7:15) {
    ParamRecov[(j - 6),1] = Nreps
    ParamRecov[(j - 6),2] = mean(Results[,j])
    ParamRecov[(j - 6),3] = mean(Results[,j]) - 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 6),4] = mean(Results[,j]) + 1.96*(sd(Results[,j])/sqrt(Nreps))
    ParamRecov[(j - 6),5] = quantile(Results[,j], c(.25))
    ParamRecov[(j - 6),6] = quantile(Results[,j], c(.50))
    ParamRecov[(j - 6),7] = quantile(Results[,j], c(.75))
  }
  
  rownamesobj = c("b0var: LM RI Variance (Outcome Scale) - Yes SMRI",
                  "evar: Total Residual Variance (Outcome Scale) - Yes SMRI",
                  "ICC: Unconditional Intra-Class Correlation - No SMRI",
                  "ICC: Unconditional Intra-Class Correlation - Yes SMRI",
                  "evar: Residual Variance (Log Scale) - Yes SMRI",
                  "b0var: LM RI Variance (Log Scale) - Yes SMRI",
                  "tau0: SM Fixed Intercept (Log Scale) - Yes SMRI",
                  "t0var: SM RI Variance (Log Scale)",
                  "b0t0corr: LM-SM RI Correlation")
  
  rownames(ParamRecov) = rownamesobj
  
  sig = sum(Results$t0var_SS == 1)
  CI = ClopPears(sig,Nreps,0.05)
  CI = as.matrix(CI)
  Power = data.frame(Power=numeric(), Lower95CI=numeric(), Upper95CI=numeric())
  Power[1,1] = mean(Results$t0var_SS)
  Power[1,2] = CI[1,1]
  Power[1,3] = CI[2,1]
}

#Provide Parameter Recovery Statistics
options(digits=2)
ParamRecov
#Power Estimates with 95% Clopper-Pearson CI
options(digits=3)
Power
